@@ -1,4 +1,6 @@
-from typing import Any, Mapping, Optional, Sequence
+from __future__ import annotations
+
+from typing import Any, Optional, Sequence
 
 from os import PathLike
 
@@ -8,12 +10,16 @@ import numpy as np
 import onnxruntime as rt
 from sklearn.base import ClassifierMixin
 
+from .translators import OnnxSeqMapDecoder, OnnxTranslator, OnnxVectorClassLabelDecoder
+from .utils import model_details
+
 class OnnxClassifier(ClassifierMixin):
     """Adapter Class for ONNX models. 
     This class loads an ONNX model and provides an interface that conforms to the scikit-learn classifier API.
     """    
 
-    def __init__(self,  model_location: "PathLike[str]") -> None:
+    def __init__(self, pred_decoder: OnnxTranslator, 
+                       proba_decoder: OnnxTranslator) -> None:
         """Initialize the model. 
         The model stored in the argument's location is loaded.
 
@@ -22,11 +28,8 @@ class OnnxClassifier(ClassifierMixin):
         model_location : PathLike[str]
             The location of the model
         """        
-        self._sess = rt.InferenceSession(model_location)
-        
-        self._input_name = self._sess.get_inputs()[0].name
-        self._label_name = self._sess.get_outputs()[0].name
-        self._proba_name = self._sess.get_outputs()[1].name
+        self.pred_decoder = pred_decoder
+        self.proba_decoder = proba_decoder
 
     def fit(self, X: np.ndarray, y: np.ndarray) -> None:
         """Fitting a model is not supported. Inference only!
@@ -55,10 +58,9 @@ class OnnxClassifier(ClassifierMixin):
         -------
         np.ndarray
             A tensor that contains the predicted classes
-        """        
-        conv_input = X.astype(np.float32)        
-        pred_onnx: np.ndarray = self._sess.run([self._label_name], {self._input_name: conv_input})[0]
-        return pred_onnx
+        """
+        translation = self.pred_decoder(X)        
+        return translation
 
     def predict_proba(self, X: np.ndarray) -> np.ndarray:
         """Return the predicted class probabilities for each input
@@ -74,39 +76,20 @@ class OnnxClassifier(ClassifierMixin):
         np.ndarray
             A probability matrix
         """        
-        conv_input = X.astype(np.float32)
-        pred_onnx = self._sess.run([self._proba_name],{self._input_name: conv_input})[0]
-        prob_vec = self._to_matrix(pred_onnx)
-        return prob_vec
-
-    def _to_matrix(self, y: Sequence[Mapping[Any, float]]) -> np.ndarray:
-        """Converts ONNX output to standard scikit-learn ``predict_proba`` 
-
-        Parameters
-        ----------
-        y : Sequence[Mapping[Any, float]]
-            A sequence of mappings of labels to floats
-
-        Returns
-        -------
-        np.ndarray
-            A probability matrix of shape (n_inputs, n_labels)
-        """        
-        if y:
-            result_matrix = np.zeros(
-                shape=(len(y), len(y[0])), 
-                dtype=np.float32)
-            
-            for i, row in enumerate(y):
-                for (lbl_idx, (lbl, proba)) in enumerate(row.items()):
-                    if isinstance(lbl, int):
-                        j = lbl
-                    else:
-                        j = lbl_idx
-                    result_matrix[i,j] = proba
-            return result_matrix
-        return np.zeros(shape=(0,0), dtype=np.float32)
+        translation = self.pred_decoder(X)        
+        return translation
 
+    @classmethod
+    def build_model(cls, model_location: "PathLike[str]") -> OnnxClassifier:
+        session = rt.InferenceSession(model_location)
+        model_details(session)
+        input_field = session.get_inputs()[0].name
+        pred_field = session.get_outputs()[0].name
+        proba_field = session.get_outputs()[1].name
+        pred_decoder = OnnxVectorClassLabelDecoder(session, input_field, pred_field)
+        proba_decoder = OnnxSeqMapDecoder(session, input_field, proba_field)
+        return cls(pred_decoder, proba_decoder)
+        
     @classmethod
     def build_data(cls, 
                    model_location: "PathLike[str]",
@@ -115,7 +98,7 @@ class OnnxClassifier(ClassifierMixin):
                    filename: "Optional[PathLike[str]]"=None, 
                    ints_as_str: bool = False,
                    ) -> il.AbstractClassifier[Any, Any, Any, Any, Any, LT, np.ndarray, np.ndarray]:
-        onnx = cls(model_location)
+        onnx = cls.build_model(model_location)
         il_model = il.SkLearnDataClassifier.build_from_model(onnx, classes, storage_location, filename, ints_as_str)
         return il_model
 
@@ -127,6 +110,6 @@ class OnnxClassifier(ClassifierMixin):
                      filename: "Optional[PathLike[str]]"=None, 
                      ints_as_str: bool = False,
                      ) -> il.AbstractClassifier[Any, Any, Any, Any, Any, LT, np.ndarray, np.ndarray]:
-        onnx = cls(model_location)
+        onnx = cls.build_model(model_location)
         il_model = il.SkLearnVectorClassifier.build_from_model(onnx, classes, storage_location, filename, ints_as_str)
-        return il_model
+        return il_model
\ No newline at end of file
